
.gs.init.plot.area <- function(xlim, labels, cex.scale = par()$cex.axis, 
	                          cex.labels, fit.labels = TRUE, 
	                          main = "gene.strip", xlab = "position", 
	                          ylab = "genes") {
	op <- par

	if(fit.labels) {                          	
		ht <- max(sapply(labels,strheight,units="inches"))
		ht.needed <- ht * length(labels)
		ht.actual <- par()$pin[2]
		if(missing(cex.labels)) {
		  if(ht.needed > ht.actual) {
		    cex.labels <- ht.actual / ht.needed
		  }
		  else {
		  	cex.labels <- par()$cex.axis
		  }
		}
		wd <- max(sapply(labels,strwidth,units="inches",cex=cex.labels))
		mai <- par()$mai
		if(is.na(par()$tck)) {
			tck <- -par()$tcl * par()$csi 
		}
		else {
			tck <- -par()$tck
		}

        	ylab.height <- strheight(ylab,"inches")
                xlab.height <- strheight(xlab,"inches")


                lm <- wd + 6 * ylab.height
                bm <-      7 * xlab.height
		mai <- par()$mai
	        mai[1]  <- bm
	        mai[2]  <- lm

		par(mai = mai)                         
        }
    
	plot(0, 0, type="n", xaxs="i", yaxt="n", bty="l", xlim=xlim, 
		ylim=c(0.5,length(labels)+0.5), cex.axis = cex.scale, 
		xlab = "", ylab= "")
	axis(2, at = (1:length(labels)), labels = as.character(labels), 
		tick = FALSE, las = 2, cex.axis = cex.labels)
	title(outer=FALSE,main = main, xlab = NULL, ylab = NULL, line=1)
	title(outer=FALSE,main = NULL, xlab = xlab, ylab = NULL, line=2)
	title(outer=FALSE,main = NULL, xlab = NULL, ylab = ylab, line=floor(par()$mai[2]/par()$csi)-1)
    par(op)
}

.gs.draw.exon <- function(gene,exon.number,col) {
    height <- 1/length(col)
    y <- gene - 0.5
	for(i in 1:length(col)) {
  	  rect(exon.number,y, exon.number + 1, y + height, col = col[i])
	  y <- y + height
    }
}

ps.value <- function(x,gps,type=c("mean-int","median-int","mean-fc","median-fc","splicing-index"),f.extra.params) {
   type = match.arg(type)
   r <- switch(type,
                "mean-int"   = mean(x[unlist(gps)]),
                "median-int" = median(x[unlist(gps)]),
                "mean-fc"    = mean(x[gps[[1]]]) - mean(x[gps[[2]]]),
                "median-fc"  = median(x[gps[[1]]]) - median(x[gps[[2]]]),
                "splicing-index" = 1)
   r
}

value.to.colour <- function(vals,col=col.rd.bl,col.range=c(-5,5)) {
  idx <- vals
  idx <- (idx - col.range[1]) /(col.range[2] - col.range[1])
  idx <- (length(col) - 1) * idx
  idx <-floor(idx) + 1
  idx[(idx < 1)] <- 1
  idx[idx > length(col)] <- length(col)
  col <- col[idx]
  col
}

.gs.draw.genes.no.introns <- function(genes,data,gps,col=col.rd.bl,col.range,type,f,f.extra.params, col.f =value.to.colour, scale.to.gene = FALSE, use.symbols = TRUE,use.mt=FALSE,no.data.col="white",main="gene.strip",xlab="exon number",ylab="genes",xlim,show.legend) {
   if(class(data) == "ExpressionSet") { data <- exprs(data) }
   if(missing(f))          { f = ps.value      }
   if(missing(gps))        { gps  = 1:dim(data)[2] }
   if(missing(type))       { 
     if(length(gps) == 2) {
	type = "mean-fc" 
     }
     else {
	type = "mean-int" 
     }
   }

   if(missing(col.range))  { 
	if((type == "mean-int") |
	   (type == "median-int")) {
	   col.range <- c(0,16)
        }
        else {
  	     col.range <- c(-5,5)
        }
    }         

   op <- par()$cex

   genes     <- unique(genes)
   all.exons <- gene.to.exon(genes,vector.out=FALSE)
   gene.data <- gene.to.exon.probeset(genes,mt=use.mt)

   ps <- gene.data[,"probeset_name"]
   keep <- ps %in% rownames(data)
   ps   <- ps[keep]
   gene.data <- gene.data[keep,]

   genes <- genes[genes %in% gene.data$"gene"]
   processed.data <- apply(data[ps,,drop=FALSE],1, f, gps=gps, type=type,f.extra.params=f.extra.params)	

   gene.data <- cbind(gene.data,"value"=processed.data)

   l   <- split(gene.data,gene.data[,1],drop=FALSE)
   l.e <- split(all.exons,all.exons[,1],drop=FALSE)
   l   <- l[genes]
   l.e <- l.e[genes]

   if(length(l) == 0) {
     stop("No exons remain after filtering. This may be because the genes to plot do not have any non-multi-target, exon probesets.")
   }

   if(use.symbols) {
     gds <- gene.details(names(l))
     rownames(gds) <- gds$"stable_id"
     gds <- gds[genes,]
     strands <- gds[,"seq_region_strand"]
     strands <- sapply(strands,function(a) {if(a > 0) { ">" } else { "<" }} ) 
     labels <- paste(gds[,"display_label"],strands)
   }
   else {
     labels <- names(l)
   }
   longest <- max(sapply(l.e, function(a) { dim(a)[1] } ))
   if(missing(xlim)) {
	xlim  <- c(1,longest+1)
   }
   .gs.init.plot.area(xlim,labels=labels,main=main,xlab=xlab,ylab=ylab)

   for(g.idx in 1:length(l)) {
     g <- l[[g.idx]]
     if(scale.to.gene) {
       gene.val <- mean(g$value)
     }

     exons <- all.exons[all.exons$"gene" == names(l)[g.idx],,drop=FALSE]
     o <- order(exons$"seq_region_start",decreasing=FALSE)
     exons <- exons[o,]
     for(e.idx in 1:dim(exons)[1]) {
	exon <- as.character(exons$"stable_id")[e.idx]
	exon.data <- g[g$"exon" == exon,]
	if(dim(exon.data)[1] < 1) {
         .gs.draw.exon(g.idx,e.idx,no.data.col)
        }
	else {
	 processed.data <- exon.data$"value"
  	 names(processed.data) <- exon.data$"probeset_name"
	 if(scale.to.gene) {
	      processed.data <- processed.data - gene.val
         }
         cols <- col.f(processed.data,col,col.range)

         .gs.draw.exon(g.idx,e.idx,cols)
        }
     }
   }
  par(cex=op)
  if(show.legend) {
     oxpd <- par()$xpd
      par(xpd=TRUE)
      w <- (par()$usr[2] - par()$usr[1]) / 2
      gene.legend(par()$usr[2] - w/5,- 2 * strheight('M'),col=col,col.range=col.range,cex=par()$cex.axis)
      par(xpd=oxpd)
  }

}

.gs.draw.genes.with.introns <- function(genes,data,gps,col=col.rd.bl,col.range,type,f,f.extra.params, col.f =value.to.colour, scale.to.gene = FALSE, use.symbols = TRUE,use.mt=FALSE,no.data.col="white",probes.min=4,main="gene.strip",xlab="nucleotide position",ylab="genes",xlim,show.legend) {
   if(class(data) == "ExpressionSet") { data <- exprs(data) }
   if(missing(f))          { f = ps.value      }
   if(missing(gps))        { gps  = 1:dim(data)[2] }
   if(missing(type))       { 
     if(length(gps) == 2) {
	type <- "mean-fc" 
     }
     else {
	type <- "mean-int" 
     }
   }
   if(missing(col.range))  { 
	if((type == "mean-int") |
	   (type == "median-int")) {
	   col.range <- c(0,16)
        }
        else {
  	     col.range <- c(-5,5)
        }
    }         


    op <- par()$cex

  genes <- unique(genes)
  all.exons <- gene.to.exon(genes,vector.out=FALSE)	
  gds       <- gene.details(unique(all.exons$"gene"))
  all.ps    <- gene.to.probeset(unique(all.exons$"gene"),vector.out=FALSE,probes.min=probes.min)
 
  if(!use.mt) {
    keep   <- !(all.ps$"probeset_name" %in% select.probewise(all.ps$"probeset_name","multitarget"))
    all.ps <- all.ps[keep,,drop=FALSE]
  }

  keep   <- all.ps$"probeset_name" %in% rownames(data)
  all.ps <- all.ps[keep,,drop=FALSE]

  processed.data <- apply(data[all.ps$"probeset_name",,drop=FALSE],1, f, gps=gps, type=type,f.extra.params=f.extra.params)	
  all.ps <- cbind(all.ps,"value"=processed.data)
 
  all.probes <- probeset.to.probe(all.ps$"probeset_name")
  all.ps <- merge(all.ps,all.probes,by="probeset_name")

#Here we need to make sure things are ordered as for the original gene list
#We also need to deal with the fact we may have lost some genes if they didn't have any non-mt probesets, for example...
  genes <- genes[genes %in% all.ps[,2]]
  rownames(gds) <- gds$"stable_id"
  gds <- gds[genes,]
  if(use.symbols) {
     strands <- gds[,"seq_region_strand"]
     strands <- sapply(strands,function(a) {if(a > 0) { ">" } else { "<" }} ) 
     labels <- paste(gds[,"display_label"],strands)
  }
  else {
    labels <- genes
  }
  longest <- max(unlist(gds$"seq_region_end") - unlist(gds$"seq_region_start"))
  if(missing(xlim)) {
    xlim <-c(1,longest + 1)
  }

  .gs.init.plot.area(xlim,labels=labels,main=main,xlab=xlab,ylab=ylab)
  l <- split(all.ps,all.ps[,2])
  l <- l[genes]
  for(g.idx in 1:length(l)) {
     g <- l[[g.idx]]

     g.start <- gds[names(l)[[g.idx]],"seq_region_start"][[1]]
     g.end   <- gds[names(l)[[g.idx]],"seq_region_end"][[1]]
     g.chr   <- gds[names(l)[[g.idx]],"name"][[1]]

     keep <- (g$"chr" == g.chr) & ((g$"seq_region_start" >= g.start)  & (g$"seq_region_start" <= g.end))
     g <- g[keep,,drop=FALSE]
     

     processed.data <- g$value
     if(scale.to.gene) {
       processed.data <- processed.data - mean(processed.data)
     }
     cols <- col.f(processed.data,col,col.range)
     rect(g$"seq_region_start"-g.start,g.idx-0.5,g$"seq_region_end"-g.start,g.idx+0.5,col=cols,border=cols)

     e.start <- unlist(all.exons[all.exons$"gene" == names(l)[[g.idx]],"seq_region_start"])
     e.end   <- unlist(all.exons[all.exons$"gene" == names(l)[[g.idx]],"seq_region_end"])

     rect(e.start-g.start,g.idx-0.1,e.end-g.start,g.idx+0.1,col="black",border="black")
  }
  par(cex=op)
  if(show.legend) {
     oxpd <- par()$xpd
      par(xpd=TRUE)
      w <- (par()$usr[2] - par()$usr[1]) / 2
      gene.legend(par()$usr[2] - w/5,- 2 * strheight('M'),col=col,col.range=col.range,cex=par()$cex.axis)
      par(xpd=oxpd)
  }
  
}

gene.strip <- function(genes,data,gps,col=col.rd.bl,col.range,type=c("mean-int","median-int","mean-fc","median-fc","splicing-index"),show.introns=FALSE,f,f.extra.params, col.f =value.to.colour, scale.to.gene = FALSE, use.symbols = TRUE,use.mt=FALSE,no.data.col="white",probes.min=4,main,xlab,ylab,xlim,show.legend=TRUE) {
  if(class(gps) != "list") { stop("gps should be a list") }
  if(missing(xlab)) {
	if(show.introns) { xlab <- "Nucleotide Position" }
        else             { xlab <- "Exon Number" }
  }
  if(missing(ylab)) { ylab <- "genes" }
  if(missing(main)) { main <- "gene.strip" }
  type <- match.arg(type)


  if(show.introns) {
     .gs.draw.genes.with.introns(genes,data,gps,col,col.range,type,f,f.extra.params,col.f, scale.to.gene, use.symbols, use.mt, no.data.col, probes.min=4,main=main,xlab=xlab,ylab=ylab,xlim,show.legend)
  }
  else {
     .gs.draw.genes.no.introns(genes,data,gps,col,col.range,type,f,f.extra.params,col.f, scale.to.gene, use.symbols, use.mt, no.data.col,main=main,xlab=xlab,ylab=ylab,xlim,show.legend)
  }


}


.gg.init.plot.area <- function(xlim, ylim,
	                       main = "", xlab = "position", 
	                       ylab = "",exon.bg.border.col) {
    
	plot(0, 0, type="n", xaxs="i", yaxs="i", bty="O", xlim=xlim, 
		ylim=ylim,
		main = main,xlab = xlab, ylab= ylab)
        abline(h=0,col=exon.bg.border.col)
}

.gg.ps.value <- function(x,gps,type=c("mean-int","median-int","mean-fc","median-fc","splicing-index")) {
   type = match.arg(type)
   r <- switch(type,
                "mean-int"   = sapply(gps,function(a) { mean(x[a]) }),
                "median-int" = sapply(gps,function(a) { median(x[a]) }),
                "mean-fc"    = mean(x[gps[[1]]]) - mean(x[gps[[2]]]),
                "median-fc"  = median(x[gps[[1]]]) - median(x[gps[[2]]]),
                "splicing-index" = 1)
   r
}



.gg <- function(gene,data,gps,gp.cols,gp.lty,scale.to.gene = FALSE, type, use.symbol = TRUE,use.mt=FALSE,probes.min=4,main="gene",xlab,ylab,xlim,ylim,exon.y,exon.height,exon.bg.col,exon.bg.border.col) {
  if(class(data) == "ExpressionSet") { data <- exprs(data) }
  if(missing(gps))        { gps  = 1:dim(data)[2] }
  if(missing(type))       { 
    if(length(gps) == 2) {
     type <- "mean-fc" 
    }
    else {
      type <- "mean-int" 
    }
  }
  if(missing(ylim))  { 
    if((type == "mean-int") |
     (type == "median-int")) {
        ylim <- c(0,16)
    }
    else {
      ylim <- c(-5,5)
    }
  }         

  if(missing(ylab))  { 
    if((type == "mean-int") |
     (type == "median-int")) {
	  ylab <- "intensity"
    }
    else {
        ylab <- "fold change"
    }
  }         

  if(length(gene)!= 1) {
   stop("Can only plot a single gene.")
  }
   
  if(missing(gp.cols)) {
    gp.cols <- 1:length(gps)
  }

  if(missing(gp.lty)) {
    gp.lty <- rep(1,length(gps))
  }

  if(missing(exon.y)) {
    if((type == "mean-int") |
     (type == "median-int")) {
      exon.y <- 0.5
    }
  }
  else {
      exon.y <- 0
  }
  
  if(missing(exon.height)) {
      exon.height <- 1
  }

  all.exons <- gene.to.exon(gene,vector.out=FALSE)	
  gds       <- gene.details(gene)
  all.ps    <- gene.to.probeset(gene,vector.out=FALSE,probes.min=probes.min)
 
  if(!use.mt) {
    keep   <- !(all.ps$"probeset_name" %in% select.probewise(all.ps$"probeset_name","multitarget"))
    all.ps <- all.ps[keep,,drop=FALSE]
  }

  keep   <- all.ps$"probeset_name" %in% rownames(data)
  all.ps <- all.ps[keep,,drop=FALSE]
  if(dim(all.ps)[1] < 1) { stop("No matching probesets found after filtering.")}
  processed.data <- apply(data[all.ps$"probeset_name",,drop=FALSE],1,.gg.ps.value, gps=gps, type=type)
  if(is.null(dim(processed.data))) {
    processed.data <- matrix(processed.data,nrow=length(processed.data))
  }
  else {
    processed.data <- t(processed.data)
  }
  if(is.null(names(gps))) {
    names(gps) <- gps
  }
  if(type == "mean-int" | type == "median-int") { 
    data.cols <- paste("gp:",names(gps))
  }
  else { 
    data.cols <- paste("gp:",names(gps),collapse=" vs. ")
  }
  colnames(processed.data)  <- data.cols
  all.ps <- cbind(all.ps,processed.data)
 
  all.probes <- probeset.to.probe(all.ps$"probeset_name")
  all.ps <- merge(all.ps,all.probes,by="probeset_name")

  if(use.symbol) {
     strand <- gds["seq_region_strand"][[1]]
     strand <- if(strand > 0) { ">" } else { "<" } 
     label  <- paste(gds["display_label"][[1]],strand)
  }
  else {
    label <- gene
  }

  if(missing(xlab)) {
    xlab <- label
  }

  if(missing(xlim)) {
    xlim <-c(gds$"seq_region_start"[[1]], gds$"seq_region_end"[[1]])
  }
  .gg.init.plot.area(xlim,ylim,main=main,xlab=xlab,ylab=ylab,exon.bg.border.col)
  pd.cols        <- colnames(all.ps) %in% data.cols
  processed.data <- all.ps[,pd.cols,drop=FALSE]
  if(scale.to.gene) {
    processed.data <- processed.data - mean(processed.data)
  }


  e.start <- unlist(all.exons[,"seq_region_start"])
  e.end   <- unlist(all.exons[,"seq_region_end"])

  if(!is.null(exon.bg.col)) {
      rect(e.start,ylim[1],e.end,ylim[2],col=exon.bg.col,border=exon.bg.border.col)
  }
  rect(e.start,exon.y-exon.height/2,e.end,exon.y+exon.height/2,col="white",border=exon.bg.border.col)
  x  <- apply(all.ps[,c("seq_region_start","seq_region_end")],1,function(a) { a[1] + (a[2] - a[1])/2 })
  x.l  <- unlist(all.ps$"seq_region_start")
  x.r  <- unlist(all.ps$"seq_region_end")
  o <- order(x,decreasing=FALSE)
  x <- x[o]
  x.l <- x.l[o]
  x.r <- x.r[o]
  for(i in 1:length(data.cols)) {  
    rect(x.l,processed.data[o,i]-exon.height/20,x.r,processed.data[o,i]+exon.height/20,border=gp.cols[i],lty=gp.lty[i])
  }

  l <- split(all.ps,all.ps$"probeset_name")
  l <- l[all.ps$"probeset_name"[o]]
  x.ps <- t(sapply(l,function(a) { c(min(a$"seq_region_start"),max(a$"seq_region_end"))}))
  for(i in 1:length(data.cols)) {  
    rect(x.ps[,1],processed.data[o,i]-exon.height/10,x.ps[,2],processed.data[o,i]+exon.height/10,border=gp.cols[i],lty=gp.lty[i])
  }
  for(i in 1:length(data.cols)) {  
    lines(x,processed.data[o,i],col=gp.cols[i],lty=gp.lty[i])
  }
  rect(xlim[1],ylim[1],xlim[2],ylim[2],border=par()$col.axis,col=NA)
}



.gg.by.order <- function(gene,data,gps,gp.cols,gp.lty,gp.pch,scale.to.gene = FALSE,type, use.symbol = TRUE,use.mt=FALSE,probes.min=4,main="gene",xlab,ylab,xlim,ylim,exon.y=0,exon.height=1,show.introns,exon.bg.col,exon.bg.border.col) {
   if(class(data) == "ExpressionSet") { data <- exprs(data) }
   if(missing(gps))        { gps  = 1:dim(data)[2] }
   if(missing(type))       { 
     if(length(gps) == 2) {
	type <- "mean-fc" 
     }
     else {
	type <- "mean-int" 
     }
   }
   if(missing(ylim))  { 
	if((type == "mean-int") |
	   (type == "median-int")) {
	   ylim <- c(0,16)
        }
        else {
  	   ylim <- c(-5,5)
        }
    }         

   if(missing(ylab))  { 
	if((type == "mean-int") |
	   (type == "median-int")) {
	     ylab <- "intensity"
        }
        else {
	     ylab <- "fold change"
        }
    }         

    if(length(gene)!= 1) {
	stop("Can only plot a single gene.")
    }
  if(!missing(gp.cols)) {
    if(length(gp.cols) != length(gps)) { 
       stop("If colours are specfied, must be the same number of colours as groups.")
    }
  }
  else {
    gp.cols <- 1:length(gps)
  }
  if(missing(gp.lty)) {
    gp.lty <- rep(1,length(gps))
  }
  if(missing(gp.pch)) {
    gp.pch <- rep(NA,length(gps))
  }
 
  all.exons <- gene.to.exon(gene,vector.out=FALSE)	
  gds       <- gene.details(gene)
  all.ps    <- gene.to.probeset(gene,vector.out=FALSE,probes.min=probes.min)
  if(!show.introns) {
    keep <-  all.ps$"probeset_name" %in% select.probewise(all.ps$"probeset_name","exonic")
    all.ps<- all.ps[keep,]
  }
  all.p     <- probeset.to.probe(all.ps$"probeset_name")
  l         <- split(all.p,all.p$"probeset_name")
  probe.region.x <- sapply(l,function(a) { median(a$"seq_region_start" + (a$"seq_region_end" - a$"seq_region_start")/2) })
  probe.region.x <- probe.region.x[all.ps$"probeset_name"]
  all.ps    <- cbind(all.ps,probe.region.x)
  if(!use.mt) {
    keep   <- !(all.ps$"probeset_name" %in% select.probewise(all.ps$"probeset_name","multitarget"))
    all.ps <- all.ps[keep,,drop=FALSE]
  }

  keep   <- all.ps$"probeset_name" %in% rownames(data)
  all.ps <- all.ps[keep,,drop=FALSE]
  if(dim(all.ps)[1] < 1) { stop("No matching probesets found after filtering.")}
  processed.data <- apply(data[all.ps$"probeset_name",,drop=FALSE],1,.gg.ps.value, gps=gps, type=type)
  if(is.null(dim(processed.data))) {
    processed.data <- matrix(processed.data,nrow=length(processed.data))
  }
  else {
    processed.data <- t(processed.data)
  }
  if(is.null(names(gps))) {
    names(gps) <- gps
  }
  if(type == "mean-int" | type == "median-int") { 
    data.cols <- paste("gp:",names(gps))
  }
  else { 
    data.cols <- paste("gp:",names(gps),collapse=" vs. ")
  }
  colnames(processed.data)  <- data.cols
  all.ps <- cbind(all.ps,processed.data)
 
  if(use.symbol) {
     strand <- gds["seq_region_strand"][[1]]
     strand <- if(strand > 0) { ">" } else { "<" } 
     label  <- paste(gds["display_label"][[1]],strand)
  }
  else {
    label <- gene
  }

  if(missing(xlab)) {
    xlab <- label
  }

  if(missing(xlim)) {
    xlim <-c(0,dim(all.ps)[1]+1)
  }
  .gg.init.plot.area(xlim,ylim,main=main,xlab=xlab,ylab=ylab,exon.bg.border.col)
  pd.cols        <- colnames(all.ps) %in% data.cols
  processed.data <- all.ps[,pd.cols,drop=FALSE]
  if(scale.to.gene) {
    processed.data <- processed.data - mean(processed.data)
  }


  e.start <- unlist(all.exons[,"seq_region_start"])
  e.end   <- unlist(all.exons[,"seq_region_end"])


  rect(e.start,exon.y-exon.height/2,e.end,exon.y+exon.height/2,col="white",border=exon.bg.border.col)
  x  <- all.ps$"probe.region.x"
  o <- order(x,decreasing=FALSE)
  x <- 1:max(o)
  if(!is.na(exon.bg.col)) {
    e.ps <- all.ps$"probeset_name" %in% select.probewise(all.ps$"probeset_name","exonic")
    e.ps <- e.ps[o]
    if(sum(e.ps) > 0) {
      if(show.introns) {
        rect(x[e.ps]-0.5,ylim[1],x[e.ps]+0.5,ylim[2],col=exon.bg.col,border=exon.bg.border.col)
      }
    }
  }

  for(i in 1:length(data.cols)) {  
    if(is.na(gp.pch[i])) {
      lines(x,processed.data[o,i],col=gp.cols[i],lty=gp.lty[i])
    }
    else {
      lines(x,processed.data[o,i],col=gp.cols[i],lty=gp.lty[i],type="b",pch=gp.pch[i])
    }
  }
  rect(xlim[1],ylim[1],xlim[2],ylim[2],border=par()$col.axis,col=NA)
}


gene.graph <- function(gene,data,gps,gp.cols,gp.lty,gp.pch,scale.to.gene = FALSE,type=c("mean-int","median-int","mean-fc","median-fc","splicing-index"), use.symbol = TRUE,use.mt=FALSE,probes.min=4,main="gene",xlab,ylab,xlim,ylim,exon.y=0,exon.height=1,by.order=FALSE,show.introns,exon.bg.col="#eeeeee",exon.bg.border.col="#dddddd") {
   type <- match.arg(type)
   if(by.order) {
     if(missing(show.introns)) {
	show.introns <- FALSE
     }
     .gg.by.order(gene,data,gps,gp.cols,gp.lty,gp.pch,scale.to.gene,type, use.symbol,use.mt,probes.min,main,xlab,ylab,xlim,ylim,exon.y,exon.height,show.introns,exon.bg.col,exon.bg.border.col)
   }
   else{    
      .gg(gene,data,gps,gp.cols,gp.lty,scale.to.gene,type, use.symbol,use.mt,probes.min,main,xlab,ylab,xlim,ylim,exon.y,exon.height,exon.bg.col,exon.bg.border.col)
   }
}


.pg.init.plot.area <- function (xlim, ylim, main = "", xlab = "position", ylab = "", 
    border.col) 
{
    plot(0, 0, type = "n", xaxs = "i", yaxt = "n", bty = "n", 
        xlim = xlim, ylim = ylim, main = main, xlab = xlab, ylab = ylab)
}

.pg.gene.height <- function(n.transcripts,transcript.height,pad) {
	pad * (3 + n.transcripts) + n.transcripts * transcript.height
	
}

.pg.set.cex <- function(label,pad) {
  h <- strheight(label)
  par(cex=pad/h)
}

.pg.gene <- function(xlim,n.transcripts,transcript.height,pad,label,col,border,text.col="black",text.bg="white",no.data.col="white") {
	h <- .pg.gene.height(n.transcripts,transcript.height,pad)
	w <- strwidth(label)
	x.pad <- strwidth("M")
	rect(xlim[1],0,xlim[2],h,col=text.bg,border=border)
	rect(xlim[1] + 2 * x.pad + w, h - 2 * pad, xlim[2] - x.pad,h - pad, col=col,border=col)
	text(xlim[1] + x.pad, h - 2 * pad,label=label,adj=c(0,0))
}

.pg.transcript <- function(xlim,transcript.no,transcript.height,pad,label,col,border,text.col="black",text.bg="white") {
	w <- strwidth(label)
	x.pad <- strwidth("M")
	bottom <- (transcript.no - 1) * (transcript.height + pad) + pad
	rect(xlim[1],bottom,xlim[2],bottom + transcript.height,col=col,border=border)
	rect(xlim[1] + x.pad, bottom + transcript.height - 2 * pad, xlim[1] + x.pad + w,bottom + transcript.height - pad, col=text.bg,border=text.bg)
	text(xlim[1] + x.pad, bottom + transcript.height - 2 * pad, label=label,adj=c(0,0))
	mid <- bottom + (transcript.height - 2 * pad)/2
	lines(c(xlim[1],xlim[2]),c(mid,mid),col=border)
}

.pg.exon <- function(xlim,transcript.no,transcript.height,pad,label,col,border,text.col="black",text.bg="white",no.data.col="white",exon.borders) {
	w <- strwidth(label)
	x.pad <- strwidth("M")
	bottom <- (transcript.no - 1) * (transcript.height + pad) + 2 * pad
	if(!exon.borders) {
      	border <- col
    }
    if(is.na(col)) {
	  col <- no.data.col
      border <- border
    }
      	
  	rect(xlim[1],bottom,xlim[2],bottom + transcript.height - 4 * pad,col=col,border=border)

	if(w < (xlim[2] - xlim[1])) {
  	  rect(xlim[1] + x.pad, bottom + transcript.height - 6 * pad, xlim[1] + x.pad + w,bottom + transcript.height - 5 * pad, col=text.bg,border=text.bg)
	  text(xlim[1] + x.pad, bottom + transcript.height - 6 * pad, label=label,adj=c(0,0))
	}
}


plot.gene <- function(gene, data, gps, scale.to.gene = FALSE, 
    type = c("mean-int", "median-int", "mean-fc", "median-fc", 
        "splicing-index"), 
    use.symbol = TRUE, use.mt = FALSE, 
    probes.min = 4, f = ps.value, f.extra.params, 
    col = col.rd.bl, col.range, col.f = value.to.colour,
    main, xlab, ylab, xlim, ylim,
    border.col = "#aaaaaa",no.data.col = "white", text.col="black",text.bg="white",
    exon.borders,
    pad=0.1,transcript.height=0.9,show.legend=TRUE) {
    	
    type <- match.arg(type)
    op <- par()$cex
	if (class(data) == "ExpressionSet") {
        data <- exprs(data)
    }
 
    if (missing(gps)) {
        gps = 1:dim(data)[2]
    }
 
    if (missing(type)) {
        if (length(gps) == 2) {
            type <- "mean-fc"
        }
        else {
            type <- "mean-int"
        }
    }
 
    if (missing(ylab)) {
       ylab <- NA
    }
    
    
    if (missing(main)) {
       main <- NA
    }
    
    if (missing(col.range)) {
      if ((type == "mean-int") | (type == "median-int")) {
         col.range <- c(0, 16)
      }
      else {
         col.range <- c(-5, 5)
      }
    }
    
    if (length(gene) != 1) {
        stop("Can only plot a single gene.")
    }

    gds           <- gene.details(gene)
    transcripts   <- gene.to.transcript(gene,vector.out=FALSE)
    n.transcripts <- dim(transcripts)[1]
    all.ps        <- gene.to.exon.probeset(gene)
    ps            <- all.ps[, "probeset_name"]
    keep <- ps %in% rownames(data)
    ps <- ps[keep]
    all.ps <- all.ps[keep,,drop=FALSE]
    processed.data <- apply(data[ps, , drop = FALSE], 1, f, gps = gps, 
        type = type, f.extra.params = f.extra.params)
    gene.val <- mean(processed.data) 
    all.ps <- cbind(all.ps, value = processed.data)
 
    if (missing(xlim)) {
       xlim <- c(gds$"seq_region_start"[[1]],gds$"seq_region_end"[[1]])
    }

    if (missing(ylim)) {
       ylim <- c(0,.pg.gene.height(n.transcripts,transcript.height,pad))
    }


    if(use.symbol) {
       strand <- gds["seq_region_strand"][[1]]
       strand <- if(strand > 0) { ">" } else { "<" } 
       label  <- paste(gds["display_label"][[1]],strand)
    }
    else {
      label <- gene
    }
    
    if(missing(xlab)) {
    	xlab <- label
    }    
    if(missing(exon.borders)) {
    	exon.borders <- (length(unique(all.ps$"exon")) < 10)
    }
    .pg.init.plot.area(xlim,ylim,main,xlab,ylab,border.col)
    .pg.set.cex(label,pad)
    .pg.gene(xlim,n.transcripts,transcript.height,pad,gene,col=col.f(gene.val,col,col.range),border=border.col,text.col=text.col,text.bg=text.bg)

    for(i in 1:dim(transcripts)[1]) {
    	xlim <- c(transcripts$"seq_region_start"[[i]],transcripts$"seq_region_end"[[i]])
        .pg.transcript(xlim,i,transcript.height,pad,transcripts$"stable_id"[[i]],col="white",border=border.col,text.col="black",text.bg="white")
        exons <- transcript.to.exon(transcripts$"stable_id"[[i]],vector.out=FALSE)
	    for(j in 1:dim(exons)[1]) {
      	 xlim <- c(exons$"seq_region_start"[[j]],exons$"seq_region_end"[[j]])
      	 processed.data <- all.ps[all.ps$"exon" == exons$"stable_id"[[j]],"value"]
         if (scale.to.gene) {

           processed.data <- processed.data - gene.val

         }
         e.col <- col.f(mean(processed.data), col, col.range)
         .pg.exon(xlim,i,transcript.height,pad,exons$"stable_id"[[j]],col=e.col,border=border.col,text.col=text.col,text.bg=text.bg,no.data.col=no.data.col,exon.borders)
        }  
    }

    par(cex=op)

    if(show.legend) {
     oxpd <- par()$xpd
      par(xpd=TRUE)
      w <- (par()$usr[2] - par()$usr[1]) / 2
      gene.legend(par()$usr[2] - w/5,- 6 * strheight('M'),col=col,col.range=col.range,cex=par()$cex.axis)
      par(xpd=oxpd)
    }

}

### map an expression level into a colour palette and get the appropriate colour out
col.rd.bl <- colorRampPalette(brewer.pal(10,"RdBu"))(11)

.value.to.colour <- function(x,cols=col.rd.bl,col.range=c(-5,5)) {
  idx <- x
  idx <- (idx - col.range[1]) /(col.range[2] - col.range[1])
  idx <- (length(cols) - 1) * idx
  idx <-floor(idx) + 1
  if(idx < 1) { idx <- 1 }
  if(idx > length(cols) ) { idx <- length(cols) }
  col <- cols[idx]
  col
}

gene.legend <- function(x,y,w,h,col,col.range,border="#dddddd",cex) {
  if(missing(w)) {
    w <- (par()$usr[2] - par()$usr[1]) / 2 / 5
  }
  if(missing(h)) {
    h <- strheight('M')
  }
  if(missing(cex)) {
    cex <- par()$cex.lab
  }

  N <- length(col)
  sub.w <- w / length(col)

  idx <- 1
  for(i in seq(sub.w,w,sub.w)) {
     rect(x+i-sub.w,y,x+i,y+h,col=col[idx],border=border)
     idx <- idx+1
  }
  tcks <- c( col.range[1],col.range[1] + (col.range[2] - col.range[1]) / 4,
             col.range[1] + (col.range[2] - col.range[1]) / 2, col.range[1] + 3 * (col.range[2] - col.range[1]) / 4, col.range[2] )

  l <- x + (w * (tcks - tcks[1]) / (col.range[2] - col.range[1]))

  text(l,y,tcks,pos=1,cex=cex)
}

