#########################################################################################
# Exonmap is a Package to provide fine grained annotation of Affymetrix exon microarrays.
# It makes use of a MySQL database containing the annotation
# 
# Please see the README.TXT file in this installation.
#
# Authors: M. Okoniewski, C.J. Miller
#########################################################################################

#### Basic function to load data, similar to simpleaffy

read.exon <- function(covdesc="covdesc",path=".",...) {
  samples <- read.AnnotatedDataFrame( paste(path,covdesc,sep="/"),sep="");
  files.to.read <- rownames(pData(samples));
  files.to.read <- paste(path,files.to.read,sep="/")
  eset <- ReadAffy(filenames=files.to.read,...);
  newPhenoData <- cbind(pData(eset),pData(samples)[rownames(pData(eset)),]);
  colnames(newPhenoData) <- c(colnames(pData(eset)),colnames(pData(samples)));
  tmp <- as.list(colnames(newPhenoData));
  names(tmp) <- colnames(newPhenoData);
  newPhenoData <- as(newPhenoData,"AnnotatedDataFrame")
  
  phenoData(eset) <- newPhenoData;	
  return(eset);
}


###
# The following functions take a character vector of names, connect to the database, request the appropriate translation,
# and return the mapped data.

#######################################################################################
probeset.to.gene <- function(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE,symbols.out=FALSE, unique=TRUE, drop.mt=FALSE) 
{
  out <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .probeset.to.gene(probeset.list[[i]])),
                  "est" = out <- rbind(out, .probeset.to.estgene(probeset.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
  else if (symbols.out) out <- out[,"display_label"]
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}


#######################################################################################
.probeset.to.gene <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToGene('",pp,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )
  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.probeset.to.estgene <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToGene('",pp,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )
  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
probeset.to.exon <- function(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE,  drop.mt=FALSE) 
{
  out <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .probeset.to.exon(probeset.list[[i]])),
                  "est" = out <- rbind(out, .probeset.to.estexon(probeset.list[[i]])),            
                  "prediction" = out <- rbind(out, .probeset.to.predexon(probeset.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}

#######################################################################################
.probeset.to.exon <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToExon('",pp,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.probeset.to.estexon <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToExon('",pp,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.probeset.to.predexon <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToPredictionExon('",pp,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else out[,"stable_id"]=out[,"prediction_exon_id"]
  out
}
#######################################################################################
probeset.to.transcript <- function(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE,  drop.mt=FALSE) 
{
  out <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .probeset.to.transcript(probeset.list[[i]])),
                  "est" = out <- rbind(out, .probeset.to.esttranscript(probeset.list[[i]])),
                  "prediction" = out <- rbind(out, .probeset.to.predtranscript(probeset.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"probeset"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}

#######################################################################################
.probeset.to.transcript <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToTranscript('",pp,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.probeset.to.esttranscript <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToTranscript('",pp,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.probeset.to.predtranscript <- function(X) 
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToPredictionTranscript('",pp,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else out[,"stable_id"]=out[,"prediction_transcript_id"]
  out
}
#######################################################################################
probeset.to.probe <- function(X, list.out=FALSE,  drop.mt=FALSE) 
{
  out <- NULL
  selected <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      if(drop.mt) probeset.list[[i]] <- exclude.probewise(probeset.list[[i]] , filter="multitarget") 
      out <- rbind(out, .probeset.to.probe(probeset.list[[i]]) )
    }
  if (list.out) out <- split(out, out[,"probeset_name"])
  out
}
#######################################################################################
.probeset.to.probe <- function(X)
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetToHit('",pp,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  out <- NULL
  if(dim(data)[1]>0)  out <- data[,]
  out
}
#######################################################################################
exon.to.transcript <- function(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  exon.list <- .split.vector(X)
  for (i in 1:length(exon.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- out <- rbind(out, .exon.to.transcript(exon.list[[i]])),
                  "est" = out <- rbind(out, .estexon.to.esttranscript(exon.list[[i]])),
                  "prediction" = out <- rbind(out, .predexon.to.predtranscript(exon.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"exon"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.exon.to.transcript <- function(X) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToTranscript('",ee,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.estexon.to.esttranscript <- function(X) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToTranscript('",ee,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.predexon.to.predtranscript <- function(X) 
{
  out <- NULL
  for (e in X)
    {
      proc <- paste ("CALL xmap_predictionExonToTranscript('",e,"')",sep="")
      outtmp <-  .doStoredProcedureFetch( proc )
      rr <- dim(outtmp)[1]
      exon <- vector(len=rr)
      exon[1:rr] <- e
      if (rr>0) outtmp <- cbind(exon, outtmp)
      out <- rbind (out, outtmp)
    }
  if(dim(out)[1]<=0) out <- NULL
  else out[,"stable_id"]=out[,"prediction_transcript_id"]
  out
}

#######################################################################################
exon.to.probeset <- function(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE,  probes.min=4) 
{
  out <- NULL
  exon.list <- .split.vector(X)
  for (i in 1:length(exon.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .exon.to.probeset(exon.list[[i]], probes.min=probes.min)),
                  "est" = out <- rbind(out, .estexon.to.probeset(exon.list[[i]], probes.min=probes.min)),
                  "prediction" = out <- rbind(out, .predexon.to.probeset(exon.list[[i]], probes.min=probes.min)))
    }
  if (list.out) out <- split(out[,"probeset_name"], out[,"exon"])
  else if (vector.out) out <- out[,"probeset_name"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.exon.to.probeset <- function(X, probes.min=4) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToProbeset('",ee,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if (dim(out)[1]>0) out <- out[out[,"num_probes"]>=probes.min,]
  out
}
#######################################################################################
.estexon.to.probeset <- function(X, probes.min=4) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToProbeset('",ee,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if (dim(out)[1]>0) out <- out[out[,"num_probes"]>=probes.min,]
  out
}
#######################################################################################
.predexon.to.probeset <- function(X, probes.min=4) 
{
  out <- NULL
  for (e in X)
    {
      proc <- paste ("CALL xmap_predictionExonToProbeset('",e,"')",sep="")
      outtmp <-  .doStoredProcedureFetch( proc )
      rr <- dim(outtmp)[1]
      exon <- vector(len=rr)
      exon[1:rr] <- e
      if (rr>0) outtmp <- cbind(exon, outtmp)
      out <- rbind (out, outtmp)
    }
  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
exon.to.gene <- function(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  exon.list <- .split.vector(X)
  for (i in 1:length(exon.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .exon.to.gene(exon.list[[i]])),
                  "est" = out <- rbind(out, .estexon.to.estgene(exon.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"exon"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.exon.to.gene <- function(X) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToGene('",ee,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.estexon.to.estgene <- function(X) 
{
  ee <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_exonToGene('",ee,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
transcript.to.gene <- function(X, db=c("core","est"), list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  transcript.list <- .split.vector(X)
  for (i in 1:length(transcript.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .transcript.to.gene(transcript.list[[i]])),
                  "est" = out <- rbind(out, .esttranscript.to.estgene(transcript.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"transcript"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.transcript.to.gene <- function(X) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToGene('",tt,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.esttranscript.to.estgene <- function(X) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToGene('",tt,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
transcript.to.exon <- function(X, db=c("core","est","prediction"), list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  transcript.list <- .split.vector(X)
  for (i in 1:length(transcript.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .transcript.to.exon(transcript.list[[i]])),
                  "est" = out <- rbind(out, .esttranscript.to.estexon(transcript.list[[i]])),
                  "prediction" = out <- rbind(out, .predtranscript.to.predexon(transcript.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"transcript"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.transcript.to.exon <- function(X) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToExon('",tt,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.esttranscript.to.estexon <- function(X) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToExon('",tt,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.predtranscript.to.predexon <- function(X) 
{
  out <- NULL
  for (t in X)
    {
      proc <- paste ("CALL xmap_predictionTranscriptToExon('",t,"')",sep="")
      outtmp <-  .doStoredProcedureFetch( proc )
      rr <- dim(outtmp)[1]
      transcript <- vector(len=rr)
      transcript[1:rr] <- t
      if (rr>0) outtmp <- cbind(transcript, outtmp)
      out <- rbind (out, outtmp)    
    }
  if(dim(out)[1]<=0) out <- NULL
  else 
   {
    out[,"stable_id"]=out[,"prediction_exon_id"]
   }
  out
}
#######################################################################################
transcript.to.probeset <- function(X, db=c("core","est","prediction"), list.out=FALSE, vector.out=TRUE, unique=TRUE,  probes.min=4) 
{
  out <- NULL
  transcript.list <- .split.vector(X)
  for (i in 1:length(transcript.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .transcript.to.probeset(transcript.list[[i]], probes.min=probes.min)),
                  "est" = out <- rbind(out, .esttranscript.to.probeset(transcript.list[[i]], probes.min=probes.min)),
                  "prediction" = out <- rbind(out, .predtranscript.to.probeset(transcript.list[[i]], probes.min=probes.min)))
    }
  if (list.out) out <- split(out[,"probeset_name"], out[,"transcript"])
  else if (vector.out) out <- out[,"probeset_name"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.transcript.to.probeset <- function(X, probes.min=4) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToProbeset('",tt,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
  out
}
#######################################################################################
.esttranscript.to.probeset <- function(X, probes.min=4) 
{
  tt <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_transcriptToProbeset('",tt,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
  out
}
#######################################################################################
.predtranscript.to.probeset <- function(X, probes.min=4) 
{
  out <- NULL
  for (t in X)
    {
      proc <- paste ("CALL xmap_predictionTranscriptToProbeset('",t,"')",sep="")
      outtmp <-  .doStoredProcedureFetch( proc )
      rr <- dim(outtmp)[1]
      transcript <- vector(len=rr)
      transcript[1:rr] <- t
      if (rr>0) outtmp <- cbind(transcript, outtmp)
      out <- rbind(out, outtmp)
    }
  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
gene.to.transcript <- function(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  gene.list <- .split.vector(X)
  for (i in 1:length(gene.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .gene.to.transcript(gene.list[[i]])),
                  "est" = out <- rbind(out, .estgene.to.esttranscript(gene.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"gene"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.gene.to.transcript <- function(X) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToTranscript('",gg,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.estgene.to.esttranscript <- function(X) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToTranscript('",gg,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
gene.to.exon <- function(X, db=c("core","est"), list.out=FALSE, vector.out=TRUE, unique=TRUE) 
{
  out <- NULL
  gene.list <- .split.vector(X)
  for (i in 1:length(gene.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .gene.to.exon(gene.list[[i]])),
                  "est" = out <- rbind(out, .estgene.to.estexon(gene.list[[i]])))
    }
  if (list.out) out <- split(out[,"stable_id"], out[,"gene"])
  else if (vector.out) out <- out[,"stable_id"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.gene.to.exon <- function(X) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToExon('",gg,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
.estgene.to.estexon <- function(X) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToExon('",gg,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
gene.to.probeset <- function(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4) 
{
  out <- NULL
  gene.list <- .split.vector(X)
  for (i in 1:length(gene.list))
    {
      db <- match.arg(db)
      r <- switch(db,
                  "core" = out <- rbind(out, .gene.to.probeset(gene.list[[i]], probes.min=probes.min)),
                  "est" = out <- rbind(out, .estgene.to.probeset(gene.list[[i]], probes.min=probes.min)))
    }
  if (list.out) out <- split(out[,"probeset_name"], out[,1])
  else if (vector.out) out <- out[,"probeset_name"]
  if (!list.out & unique) out <- unique(out)
  out
}
#######################################################################################
.gene.to.probeset <- function(X, probes.min=4) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToProbeset('",gg,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
  out
}
#######################################################################################
.estgene.to.probeset <- function(X, probes.min=4) 
{
  gg <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_geneToProbeset('",gg,"')",sep="")
  out <-  .doStoredProcedureOtherFeaturesFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  else if(dim(out)[1]>0) out <- out[out[,"probe_count"]>=probes.min,]
  out
}
#######################################################################################
symbol.to.gene <- function(X) {
  if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
  if(!exists(".ExonmapConnection")) {
    stop("Not connected to database."); 
  }
  ss <- .vector.to.commalist(X)
  if (length(grep("homo",.ExonmapDBName))>0) num.spe=1100
  if (length(grep("mus",.ExonmapDBName))>0) num.spe=1400
  if (length(grep("rattus",.ExonmapDBName))>0) num.spe=2800
  proc <- paste ("CALL xmap_externalDbidToGene(",num.spe,",'",ss,"')",sep="")
  out <-  .doStoredProcedureFetch( proc )

  if(dim(out)[1]<=0) out <- NULL
  out
}
#######################################################################################
symbol.to.probeset <- function(X, vector.out=TRUE, symbol.db=c("hugo","symbol","UniProt","RefSeqPeptide","RFAM","UniProtSpliceVariant","miRNA"),probes.min=4)
{
  gg <- .vector.to.commalist(X)
  db <- match.arg(symbol.db)
  proc <- switch(db,
                 "hugo"                 = paste("CALL xmap_externalDbidToProbeset(1100,'",gg,"')",sep=""), 
                 "symbol"               = paste("CALL xmap_externalDbidToProbeset(1100,'",gg,"')",sep=""),
                 "UniProt"              = paste("CALL xmap_externalDbidToProbeset(2000,'",gg,"')",sep=""),
                 "RefSeqPeptide"        = paste("CALL xmap_externalDbidToProbeset(1815,'",gg,"')",sep=""),
                 "RFAM"                 = paste("CALL xmap_externalDbidToProbeset(4200,'",gg,"')",sep=""),
                 "UniProtSpliceVariant" = paste("CALL xmap_externalDbidToProbeset(2202,'",gg,"')",sep=""),
                 "miRNA"                = paste("CALL xmap_externalDbidToProbeset(3300,'",gg,"')",sep=""))
  data <-  .doStoredProcedureFetch( proc )

  out <- NULL
  if(dim(data)[1]>0)
    {
      idx <-  which (data[,4]>=probes.min)
      if(vector.out) out <- data[idx,3]
      else out <- data[idx,]
    }
  out
}      

#######################################################################################
.exclude.probesets <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget")) 
{
  out <- NULL
  selected <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      selected <- c(selected, .select.X(probeset.list[[i]],filter=filter))
    }
  out <- setdiff(X , selected)
  out
}
#######################################################################################
                                        # old .select.probesets
.select.X <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget")) 
{
  out <- NULL
  list.in <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetStats('",list.in,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  if(dim(data)[1]>0)
    {
      if(filter=="intronic") idx <- which(data[,"total_distinct_genes"]>0 & data[,"total_distinct_exons"]==0)
      if(filter=="exonic") idx <- which(data[,"total_distinct_genes"]>0 & data[,"total_distinct_exons"]>1)
      if(filter=="plainexonic") idx <- which(data[,"total_distinct_genes"]==1 & data[,"total_distinct_exons"]==1)
      if(filter=="plainintronic") idx <- which(data[,"total_distinct_genes"]==1 & data[,"total_distinct_exons"]==0)
      if(filter=="plain") idx <- which(data[,"total_distinct_genes"]==1)
      if(filter=="nonexonic") idx <- which(data[,"total_distinct_exons"]==0)
      if(filter=="intergenic") idx <- which(data[,"total_distinct_genes"]==0)
      if(filter=="multitarget") idx <- which( data[,"total_distinct_genes"]>1 | data[,"total_distinct_exons"]>1 )
    }
  out <- data[idx,"probeset"]
  out
}
#######################################################################################
.select.probesets <- function(X,filter=c("nonexonic","intronic","intergenic","multitarget")) 
{
  out <- NULL
  selected <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      out <- c(out, .select.X(probeset.list[[i]],filter=filter))
    }
  out
}
#######################################################################################
exclude.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget"), mt.level=1) 
{
  out <- NULL
  selected <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      selected <- c(selected, .select.probewise(probeset.list[[i]],filter=filter, mt.level=mt.level))
    }
  out <- setdiff(X , selected)
  out
}
#######################################################################################
.select.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget","nulltarget"), mt.level=1) 
{
  out <- NULL
  list.in <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetSummary('",list.in,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  if(dim(data)[1]>0)
    {   
      if(filter=="intronic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]==0  & data[,"geneScore"]>0),"probeset"]
      if(filter=="exonic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]>0  & data[,"geneScore"]>0),"probeset"]
      if(filter=="intergenic") out <- data[(data[,"hitScore"]==1 & data[,"exonScore"]==0  & data[,"geneScore"]==0),"probeset"]
      if(filter=="multitarget") out <- data[which( data[,"hitScore"]>mt.level),"probeset"]
      if(filter=="nulltarget") out <- data[( data[,"hitScore"]==0),"probeset"]
    }
  out
}
#######################################################################################
select.probewise <- function(X,filter=c("exonic","intronic","intergenic","multitarget"), mt.level=1) 
{
  out <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      out <- c(out, .select.probewise(probeset.list[[i]],filter=filter, mt.level=mt.level))
    }
  out
}
#######################################################################################
.probeset.stats <- function(X)
{
  pp <- .vector.to.commalist(X)
  proc <- paste ("CALL xmap_probesetSummary('",pp,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  data
}
#######################################################################################
probeset.stats <- function(X)
{
  out <- NULL
  probeset.list <- .split.vector(X)
  for (i in 1:length(probeset.list))
    {
      out <- rbind(out, .probeset.stats(probeset.list[[i]]))
    }
  out
}
#######################################################################################
probesets.in.range <- function(start, stop, strand, chr)
{
  out <- NULL

  proc <- paste ("CALL xmap_probesetsInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  if(dim(data)[1]>0) out <- data[,"name"]
  out
}
#######################################################################################
exons.in.range  <- function(start, stop, strand, chr)
{
  out <- NULL
  proc <- paste("CALL xmap_exonsInRange(", start, ",",
                stop, ",", strand, ",'", chr, "')", sep = "")
  data <- .doStoredProcedureFetch(proc)
  if (dim(data)[1] > 0)
    out <- data[, "stable_id"]

  out
}
#######################################################################################
transcripts.in.range  <- function(start, stop, strand, chr)
{
  out <- NULL
  proc <- paste("CALL xmap_transcriptsInRange(", start, ",",
                stop, ",", strand, ",'", chr, "')", sep = "")
  data <- .doStoredProcedureFetch(proc)
  if (dim(data)[1] > 0)
    out <- data[, "stable_id"]

  out
}
#######################################################################################
genes.in.range <- function(start, stop, strand, chr)
{
  out <- NULL
  proc <- paste ("CALL xmap_genesInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  if(dim(data)[1]>0) out <- data
  out
}
#######################################################################################
probes.only.in.range <- function(start, stop, strand, chr)
{
  out <- NULL
  proc <- paste ("CALL xmap_probesOnlyInRange(",start,",",stop,",",strand,",'", chr,"')",sep="")
  data <-  .doStoredProcedureFetch( proc )

  if(dim(data)[1]>0) out <- data
  out
}

#######################################################################################
gene.details <- function(X) {
  out <- NULL
  data <-  sapply(X,function(a) {
    proc <- paste("CALL xmap_geneDetails('",a,"')",sep="")

    r    <- .doStoredProcedureFetch( proc )
    
    if(dim(r)[1] == 0) { stop(paste("Could not find gene details for '",a,"'.",sep=""))}
    r
  })
  data <- t(data)
  rownames(data) <- X
  if(dim(data)[1]>0) out <- as.data.frame(data)
  out
}

######################################################################################
transcript.details <- function(X) {
  out <- NULL
  data <-  sapply(X,function(a) {
    proc <- paste("CALL xmap_transcriptDetails('",a,"')",sep="")

    r    <- .doStoredProcedureFetch( proc )

    if(dim(r)[1] == 0) { stop(paste("Could not find transcript details for '",a,"'.",sep=""))}
    r
  })
  data <- t(data)
  rownames(data) <- X
  if(dim(data)[1]>0) out <-  as.data.frame(data)
  out
}

#######################################################################################
exon.details <- function(X) {
  out <- NULL
  data <- c()
  for(i in X) {
    proc <- paste("CALL xmap_exonDetails('",i,"')",sep="")

    r    <- .doStoredProcedureFetch( proc )

    if(dim(r)[1] == 0) { stop(paste("Could not find exon details for '",a,"'.",sep=""))}
    data <- rbind(data,r)
  }
  if(dim(data)[1]>0) out <-  as.data.frame(data)
  out
}

#######################################################################################
probesets.in.eset <- function(X,data) {
  if(class(data) != "ExpressionSet") { stop("data should be an ExpressionSet") }
  intersect(X,rownames(exprs(data)))
}

#######################################################################################
                                        # utility functions - hidden
.vector.to.commalist <- function(v)
{
  out <- v[1]
  if(length(v)>1)
    {
      for (i in 2:length(v))
        {
          out <- paste(out,v[i], sep=",")
        }
    }
  out
}

.exit.cleanup <- function(connection) {
  res <-  dbListResults(connection)
  for(i in res)  {
    if(!dbHasCompleted(i)) {
      fetch(i,-1)
      dbClearResult(i)
    }
  }
  while(dbMoreResults(connection)) {
    res <- dbNextResult(connection)
    if(!dbHasCompleted(res)) {
      fetch(res,-1)
      dbClearResult(res)
    }
  }
}



.doStoredProcedureFetch <- function( sql ) {
  if(is.null(.ExonmapHost)) {stop("Xmap database not initialized yet!")}
  if(!exists(".ExonmapConnection")) {
    stop("Not connected to database."); 
  }
  else {
    connection <- .ExonmapConnection
  }

  on.exit( .exit.cleanup(connection))

  res <- dbSendQuery( connection, sql )
  data <- fetch(res,-1)
  while(dbMoreResults(connection)) {
    res <- dbNextResult(connection)
    if(!dbHasCompleted(res)) {
      fetch(res,-1)
    }
  }
  dbClearResult(res)
  data
}

.split.vector <- function(v)
{
  out <- NULL
  if(length(v)>0)
    {
      extrabit <- 0
      elt.size <- max(nchar(v))

      list.size <- trunc(65000/(elt.size+1))

      num.lists <- trunc(length(v)/list.size)

      if((length(v)%%list.size)>0) {extrabit <- 1}
      if(num.lists>0)
        {
          for (i in 0:(num.lists-1))
            {

              out[[i+1]]<- v[((i*list.size)+1):((i+1)*(list.size))]
            }
        }

      if(extrabit==1) out[[num.lists+1]] <- v[(num.lists*list.size+1):(length(v))]
    }
  else out[[1]] <- v
  out
}

#######################################################################################
# Database switching
xmapDatabase <- function(db=c("Human", "Mouse","Rat"))
{
  .xmapDisconnect()
  db <- match.arg(db)
  switch(db,
         "Human" = .dbSetup("human.mycnf"),
         "Mouse" = .dbSetup("mouse.mycnf"),
         "Rat"   = .dbSetup("rat.mycnf"))
  .xmapConnect()
  cat("done.\n");
}

.xmapConnect <- function()
{
  con <- dbConnect("MySQL",user=.ExonmapUser, password=.ExonmapPassword, dbname=.ExonmapDBName, host=.ExonmapHost, client.flag=CLIENT_MULTI_STATEMENTS )
  assign(".ExonmapConnection",con,envir=.GlobalEnv);
}


.xmapDisconnect <- function() {
  if(exists(".ExonmapConnection")) {
    dbDisconnect(.ExonmapConnection)
    remove(.ExonmapConnection,envir=.GlobalEnv)
  }
}



.dbSetup <- function(file) {
  dir <- Sys.getenv("R_XMAP_CONF_DIR")
  if(dir == "") {
     warning("Don't know where configuration files are: environment variable R_XMAP_CONF_DIR is not set. trying '~/.exonmap'.")
     dir <- "~/.exonmap"
  }
  
  full.path <- file.path(dir,file)
  
  if(file.exists(full.path)) {
    conf <- read.table(full.path, row.names=1)
    assign(".ExonmapUser",as.character(conf["user",]),envir=.GlobalEnv);
    assign(".ExonmapPassword", as.character(conf["password",]),envir=.GlobalEnv);
    assign(".ExonmapDBName",as.character(conf["database",]),envir=.GlobalEnv);
    assign(".ExonmapHost",as.character(conf["host",]),envir=.GlobalEnv);
    assign(".ExonmapSpecies",as.character(conf["species",]),envir=.GlobalEnv);
  }
  else stop(paste("Couldn't find file:",full.path))
}  

showXmapDatabase <- function()
{
  cat("User",.ExonmapUser,"\n" )
  cat("Password",.ExonmapPassword,"\n" )
  cat("Database",.ExonmapDBName,"\n")
  cat("Host",.ExonmapHost,"\n")
  cat("OtherFeaturesDB",.ExonmapOtherFeatures,"\n")
}

#####################################################################################
                                        #Other features database functions                                                  #
#####################################################################################

.doStoredProcedureOtherFeaturesFetch <- function( sql ) {
  if(is.null(.ExonmapOtherFeatures)) {stop("Xmap other features database not initialized yet!")}
  if(!exists(".ExonmapOtherFeaturesConnection")) {
    stop("Not connected to Other Features database."); 
  }
  else {
    connection <- .ExonmapOtherFeaturesConnection
  }
  
  on.exit( .exit.cleanup(connection))
  
  res <- dbSendQuery( connection, sql )
  data <- fetch( res,-1) 
  while(dbMoreResults(connection)) {
    res <- dbNextResult(connection)
    if(!dbHasCompleted(res)) {
      fetch( res,-1)
    }
  }
  dbClearResult(res)
  data
}

xmapExtras <- function(db=NULL)
{
  if (is.null(.ExonmapDBName)) stop("Main database not initialized yet!")  
  if (is.null(db))
    {
      tmp <- strsplit(.ExonmapDBName, "core")[[1]]
      db <- paste(tmp[1],"otherfeatures",tmp[2], sep="")
      cat ("Assigned ",db," as other features database name. Please make sure it is correct.\n" )
    }
  assign(".ExonmapOtherFeatures",db,envir=.GlobalEnv);
  .xmapOtherFeaturesConnect()
}


.xmapOtherFeaturesConnect <- function()
{
  con <- dbConnect("MySQL",user=.ExonmapUser, password=.ExonmapPassword, dbname=.ExonmapOtherFeatures, host=.ExonmapHost, client.flag="196608" )
  assign(".ExonmapOtherFeaturesConnection",con,envir=.GlobalEnv);
}

.xmapOtherFeaturesDisconnect <- function() {
  if(exists(".ExonmapOtherFeaturesConnection")) {
    dbDisconnect(.ExonmapOtherFeaturesConnection)
  }
}







