\name{probeset.to.probe}
\alias{probeset.to.probe}
\alias{probeset.to.exon}
\alias{probeset.to.transcript}
\alias{probeset.to.gene}
\alias{exon.to.probeset}
\alias{exon.to.transcript}
\alias{exon.to.gene}
\alias{transcript.to.gene}
\alias{transcript.to.exon}
\alias{transcript.to.probeset}
\alias{gene.to.transcript}
\alias{gene.to.exon}
\alias{gene.to.probeset}
\alias{symbol.to.probeset}
\alias{symbol.to.gene}

\title{ Use the X:MAP database to translate between genes, transcripts, exons, probesets and probes }
\description{
Each of the functions is in the form X.to.Y, and takes a list of identifiers for the things of type X and returns the corresponding things of type Y.

}
\usage{
  probeset.to.probe(X,list.out=FALSE, drop.mt=FALSE)
  probeset.to.exon(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE, drop.mt=FALSE) 
  probeset.to.transcript(X, db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE, drop.mt=FALSE) 
  probeset.to.gene(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE,symbols.out=FALSE, unique=TRUE, drop.mt=FALSE) 
  exon.to.probeset(X, db=c("core","est","prediction"), list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4) 
  exon.to.transcript(X,db=c("core","est","prediction"),list.out=FALSE, vector.out=TRUE, unique=TRUE) 
  exon.to.gene(X,db=c("core","est"),list.out=FALSE, vector.out=TRUE, unique=TRUE) 
  transcript.to.exon(X, db=c("core","est","prediction"), list.out=FALSE,vector.out=TRUE, unique=TRUE) 
  transcript.to.probeset(X, db=c("core","est","prediction"),list.out=FALSE,vector.out=TRUE, unique=TRUE, probes.min=4)
  transcript.to.gene (X,db=c("core","est"), list.out=FALSE, vector.out=TRUE, unique=TRUE) 
  gene.to.exon(X, db=c("core","est"),list.out=FALSE, vector.out=TRUE, unique=TRUE)   
  gene.to.probeset(X, db=c("core","est"), list.out=FALSE, vector.out=TRUE, unique=TRUE, probes.min=4) 
  symbol.to.probeset(X, vector.out=TRUE,symbol.db=c("hugo","symbol","UniProt","RefSeqPeptide","RFAM","UniProtSpliceVariant","miRNA"),probes.min=4)
  symbol.to.gene(X) 
}

\arguments{
  \item{X}{ Vector of identifiers to map (e.g. an ensembl gene id) }
  \item{db}{ Return data from the Ensembl gene database (default), from genscan predictions or from the EST subset of Ensembl?}
  \item{list.out}{ If TRUE, results are presented as a list. Overrides effects of vector.out and symbols.out }
  \item{vector.out}{ By default, return only the names of the mapped identifiers, if FALSE, return a summary table }
  \item{symbols.out}{ If TRUE, returns just gene symbols }
  \item{unique}{ If TRUE, pre-filters the data to remove dublicates }
  \item{drop.mt}{ Filter out multi-targeting probesets prior to database search }
  \item{probes.min}{ Only return probesets that hit the genome with at least this number of probes }
  \item{symbol.db}{ The name of the database containing the gene symbol
    to map to probeset or gene (e.g. hugo, UniProt, etc.) }
  
}
\details{
  Connects to the X:MAP database to retrieve data. For Ensembl genes and
  genscan predictions, xmapDatabase() must be called first. ESTs are
  stored in a separate 'extras' database. To use the EST data xmapExtras() must be called in
  addition to xmapDatabase(), to set up the connection.
}

\value{
  Vector of probeset names, or a data frame with probeset names and number of probe hits.
}

\seealso{\code{\link{xmapDatabase}} \code{\link{xmapExtras}}}


\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Michal Okoniewski }

\keyword{ misc }
\examples{ 
  \dontrun{
library(exonmap)
#reading arrays etc.
#x  <- read.exon()

data(exonmap)
xmapDatabase("Human")

cmp <- pc(x.rma,"group",c("a","b"))

fc.r <-fc(cmp)
tt.r <-tt(cmp)

# Only a test! - and x doesn't have many genes in it!
keep.fc <- abs(fc.r) > 1
keep.tt <- tt.r < 0.05

keep <- keep.fc & keep.tt

ps <- featureNames(x.rma)[keep]

# OK, test mapping functions

ex <- probeset.to.exon(ps)
tr <- probeset.to.transcript(ps)
ge <- probeset.to.gene(ps)


probeset.to.exon(ps,list.out=TRUE)
probeset.to.transcript(ps,list.out=TRUE)
probeset.to.gene(ps,list.out=TRUE)

exon.to.transcript(ex)
exon.to.gene(ex)
exon.to.probeset(ex)

transcript.to.gene(tr)
transcript.to.exon(tr)
transcript.to.probeset(tr)

gene.to.transcript(ge)
gene.to.exon(ge)
gene.to.probeset(ge)

gds <- gene.details(ge)
trs <- transcript.details(tr)
exs <- exon.details(ex)
pss <- probeset.stats(ps)

symbol.to.gene(unlist(gds[,13]))
symbol.to.probeset(unlist(gds[,13]))


genes.in.range(1,100000,1,1)
genes.in.range(1,100000,-1,1)

transcripts.in.range(1,100000,1,1)
transcripts.in.range(1,100000,-1,1)

exons.in.range(1,100000,1,1)
exons.in.range(1,100000,-1,1)

probesets.in.range(1,100000,1,1)
probesets.in.range(1,100000,-1,1)

probeset.stats(ps)
}
}
