\name{xmapBridge}
\alias{xmapBridge}
\alias{getBridgeFile}

\title{ Generate a data file which can be displayed in the X:MAP browser
  using the X:MAP bridge }

\description{
  Writes out a datafile containing expression data for the gene of
  interest. This file is stored in a cache directory specified by the
  environment \code{R\_XMAP\_BRIDGE\_CACHE}. When the xmapBridge is running
  locally, the file will be detected, and can be displayed in the X:MAP
  genome browser.

  Note that in order to use this function, the line species=homo\_sapiens
  (or mus\_musculus or rattus\_norvegicus) must be specified as an additional
  line in the .mycnf file for the appropriate database. See the exonmap
  installation instructions for more details.
}

\usage{
  xmapBridge(x,eset,colours,bridge.file=getBridgeFile(x),description="exonmap expression data",gps,type=c("fc","int"),overwrite=FALSE,show.introns=TRUE,show.mt=FALSE,open.browser=FALSE,ylim)
  getBridgeFile(basename)
}

\arguments{
  \item{x}{ Ensembl gene identifier for the gene to plot }
  \item{eset}{\code{\link[affy]{ExpressionSet}} object containing
    expression data to plot}
  \item{colours}{ Can be used to specify explicitly what colours to plot the
    expression data in the browser. If specified, must be a character
    vector of hexadecimal strings (e.g. \#0000FF) the same length as
    there are groups in the parameter 'gps'. }
  \item{bridge.file}{Where should the output file be written. By
    default, the filename is generated by
    \code{\link[exonmap]{getBridgeFile}}.}
  \item{description}{free text description of this expression dataset -
    is shown in the browser}
  \item{gps}{A list of vectors providing indices into the expression data to specify how samples
    should be grouped together. If fold change data are plotted, then
    there must be exactly two groups. For example
    gps=list(cellLineA=1:3,cellLineB=4:6) would specify two groups,
    called cellLineA and cellLineB, each consisting of three samples
    (1,2,3 and 4,5,6).}
  \item{type}{The type of plot to draw. fc specifies fold change, int,
    intensity.}
  \item{overwrite}{Unless TRUE the function will not overwrite an
    existing data file}
  \item{show.introns}{If FALSE, plot only exon data.}
  \item{show.mt}{If TRUE, show data for multi-target probesets}
  \item{open.browser}{If TRUE, attempt to open the default web browser
    showing X:MAP centered on the specified gene.}
  \item{ylim}{Can be used to specify the Y bounds of the plot. By
    default they are c(0,14) for intensity, c(-5,5) for fold
    change. Data that exceed these limits are truncated to the maximum
    or minimum values. }
  \item{basename}{base filename from which to construct the full path to
    an xmap bridge file in the cache directory.}
}

\details{
  For more details of installation and operation, see the package
  vignette.
  
}

\seealso{

}
\value{
}
\references{\url{ http://bioinformatics.picr.man.ac.uk/}}
\author{ Crispin Miller }


\keyword{ misc }
\examples{ 
  \dontrun{
    xmapBridge("ENSG00000000003",eset=x,gps=list(A=1:3,B=4:6),type="fc",overwrite=FALSE)
  }
}
